local MainWindow, BuyPanel, SellPanel, SellPanelWindow
local Current = {}
Market = {}
Market.opcode = 15
Market.feePercent = 0.1
Market.time = {
    {value = 60 * 60 * 24 * 30, name = "30 dias"}
}
Market.category = {
    todos = 1
    -- Utilities = 2,
    -- Addons = 3,
    -- Consumable = 4,
    -- Events = 5,
    -- Evolution = 6,
    -- Foods = 7,
    -- Furnitures = 8,
    -- ["General Itens"] = 9,
    -- ["Legend Coins"] = 10,
    -- Lures = 11,
    -- Podex = 12,
    -- ["Pok Balls"] = 13,
    -- ["Pokmon"] = 14,
    -- Utilities = 15
    -- consumables = 2,
}
Market.order = {
    timedesc = 1,
    timeasc = 2,
    itemdesc = 3,
    itemasc = 4,
    namedesc = 3,
    nameasc = 4,
    sellerdesc = 5,
    sellerasc = 6,
    amountdesc = 7,
    amountasc = 8,
    pricedesc = 9,
    priceasc = 10
}

local function onGameStart()
    if not MainWindow then
        return
    end
    MainWindow:hide()
end

local function onGameEnd()
    if not MainWindow then
        return
    end
    closeMarket()
end

local function connecting(gameEvent)
    if gameEvent then
        connect(
            g_game,
            {
                onGameEnd = onGameEnd,
                onGameStart = onGameStart,
                onWalk = closeMarket,
                onAutoWalk = closeMarket
            }
        )
    end
    return true
end

local function disconnecting(gameEvent)
    if gameEvent then
        connect(
            g_game,
            {
                onGameEnd = onGameEnd,
                onGameStart = onGameStart,
                onWalk = closeMarket,
                onAutoWalk = closeMarket
            }
        )
    end
    return true
end

function init()
    connecting(true)
    MainWindow = g_ui.loadUI("market", modules.game_interface.getRootPanel())
    BuyPanel = MainWindow:getChildById("buyPanel")
    SellPanel = MainWindow:getChildById("sellPanel")
    SellPanelWindow = g_ui.createWidget("SellPanelWindow", modules.game_interface.getRootPanel())
    HistoricPanel = MainWindow:getChildById("historicPanel")
    BuyNowWindow = g_ui.createWidget("BuyNowWindow", modules.game_interface.getRootPanel())

    local options = {}
    for category, id in pairs(Market.category) do
        table.insert(options, {option = category, id = id})
    end
    table.sort(
        options,
        function(a, b)
            return a.id < b.id
        end
    )
    for _, tab in pairs(options) do
        BuyPanel:getChildById("comboBox"):addOption(tab.option:sub(1, 1):upper() .. tab.option:sub(2), _)
    end
    for _, _time in ipairs(Market.time) do
        SellPanelWindow:getChildById("sell_timebox"):addOption(_time.name)
    end
    BuyPanel:getChildById("comboBox").onOptionChange = getBuyItems
    if g_game.isOnline() then
        onGameStart()
    end
    ProtocolGame.registerExtendedJSONOpcode(Market.opcode, parseMarket)
end

function terminate()
    -- Removing the connectors
    disconnecting(true)
    ProtocolGame.unregisterExtendedJSONOpcode(Market.opcode)

    if BuyPanel.event then
        removeEvent(BuyPanel.event)
        BuyPanel.event = nil
    end
    if SellPanel.event then
        removeEvent(SellPanel.event)
        SellPanel.event = nil
    end
    BuyNowWindow:destroy()
    BuyNowWindow = nil
    SellPanelWindow:destroy()
    SellPanelWindow = nil
    --BidWindow:destroy()
    --BidWindow = nil
    -- MainButton:destroy()
    -- MainButton = nil
    MainWindow:destroy()
    MainWindow:ungrabKeyboard()
    MainWindow = nil
end

function toggle()
    if MainWindow:isVisible() then
        -- MainButton:setOn(false)
        hide()
    else
        getBuyItems()
    end
end

function show()
    BuyNowWindow:hide()
    --BidWindow:hide()
    MainWindow:show()
    MainWindow:grabKeyboard()
end

function hide()
    if BuyPanel.event then
        removeEvent(BuyPanel.event)
        BuyPanel.event = nil
    end
    if SellPanel.event then
        removeEvent(SellPanel.event)
        SellPanel.event = nil
    end
	if SellPanelWindow then
		SellPanelWindow:hide()
	end
    MainWindow:hide()
    MainWindow:ungrabKeyboard()
end

function togglePage(id)
    if id == "firstPage" then
        Current.page = 1
    elseif id == "prevPage" then
        Current.page = math.max(1, Current.page - 1)
    elseif id == "nextPage" then
        Current.page = math.min(Current.max_page, Current.page + 1)
    elseif id == "lastPage" then
        Current.page = Current.max_page
    end
    getBuyItems()
end

function getBuyItems(reset)
    if reset then
        Current.page = 1
        Current.order = Market.order.timeasc
    end

    local params = {
        protocol = "buy_items",
        category = BuyPanel:getChildById("comboBox"):getCurrentOption().data,
        page = Current.page or 1,
        order = Current.order or Market.order.timeasc,
        search_string = Current.search_string
    }
    g_game.getProtocolGame():sendExtendedOpcode(Market.opcode, json.encode(params))
    MainWindow:grabKeyboard()
end

function getSellItems()
    local params = {
        protocol = "sell_items"
    }
    g_game.getProtocolGame():sendExtendedOpcode(Market.opcode, json.encode(params))
end

function onTextChange()
    Current.search_string = BuyPanel:getChildById("searchEdit"):getText()
end

function onSellPriceChange()
    local value = math.max(1, SellPanelWindow:getChildById("sell_amount"):getValue())
    local priceNumber = tonumber(SellPanelWindow:getChildById("sell_priceedit"):getText())
    if priceNumber then
        SellPanelWindow:getChildById("sell_value"):setText(
            "Preo: $" ..
                comma_value((priceNumber * value)) ..
                    "\nTaxa: $" ..
                        comma_value(getMarketFee(priceNumber * value)) ..
                            "\nTotal: $" .. comma_value((priceNumber * value) + getMarketFee(priceNumber * value))
        )
    else
        SellPanelWindow:getChildById("sell_value"):setText("")
    end
end

function onSellBidCheckChange()
    SellPanel:getChildById("bidEdit"):setEnabled(SellPanel:getChildById("bidCheckBox"):isChecked())
end

function cancelSellItem(widget)
    local params = {
        protocol = "cancelsell_item",
        item_code = widget.market_item.item_code
    }
    g_game.getProtocolGame():sendExtendedOpcode(Market.opcode, json.encode(params))
end

function showInfo(widget, mousePos, mouseButton, market_item)
    if mouseButton == MouseRightButton then
        local params = {protocol = "info", item_code = market_item.item_code}
        local menu = g_ui.createWidget("PopupMenu")
        menu:addOption(
            tr("Info"),
            function()
                g_game.getProtocolGame():sendExtendedOpcode(Market.opcode, json.encode(params))
            end
        )
        menu:display(mousePosition)
    end
end

function buyItem()
    if not Current.buyWidget then
        return
    end
    local rowItem = BuyNowWindow:getChildById("item")
    rowItem:setItemId(Current.buyWidget.market_item.clientId)
    rowItem.onHoverChange = function(self, hovered)
        if hovered then
        else
        end
    end
    rowItem:setItemCount(Current.buyWidget.market_item.count)
    local rowScrollbar = BuyNowWindow:getChildById("countScrollBar")
    rowScrollbar:setMinimum(1)
    rowScrollbar:setMaximum(Current.buyWidget.market_item.count)
    rowScrollbar:setText(Current.buyWidget.market_item.count)
    rowScrollbar:setValue(Current.buyWidget.market_item.count)

    local rowPrice = BuyNowWindow:getChildById("price")
    rowPrice:setText("$" .. comma_value(Current.buyWidget.market_item.price * Current.buyWidget.market_item.count))

    local rowSpinBox = BuyNowWindow:getChildById("spinBox")
    rowSpinBox:setMinimum(0)
    rowSpinBox:setMaximum(Current.buyWidget.market_item.count)
    rowSpinBox:setValue(0)
    rowSpinBox:hideButtons()
    rowSpinBox:focus()
    rowSpinBox.firstEdit = true
    local onSpinBoxValueChange = function(self, value)
        rowSpinBox.firstEdit = false
        rowScrollbar:setValue(value)
    end
    rowSpinBox.onValueChange = onSpinBoxValueChange

    local refresh = function()
        if rowSpinBox.firstEdit then
            rowSpinBox:setValue(rowSpinBox:getMaximum())
            rowSpinBox.firstEdit = false
        end
    end

    rowScrollbar.onValueChange = function(self, value)
        rowItem:setItemCount(value)
        rowScrollbar:setText(value)
        rowPrice:setText("$" .. comma_value(value * Current.buyWidget.market_item.price))
        rowSpinBox.onValueChange = nil
        rowSpinBox:setValue(value)
        rowSpinBox.onValueChange = onSpinBoxValueChange
    end

    local okButton = BuyNowWindow:getChildById("buttonOk")
    local buyFunc = function()
        local params = {
            protocol = "buy_item",
            item_code = Current.buyWidget.market_item.item_code,
            itemid = Current.buyWidget.market_item.itemid,
            buy_count = rowScrollbar:getValue()
        }
        g_game.getProtocolGame():sendExtendedOpcode(Market.opcode, json.encode(params))
        show()
    end
    local cancelButton = BuyNowWindow:getChildById("buttonCancel")
    local cancelFunc = function()
        show()
    end

    BuyNowWindow.onEnter = buyFunc
    BuyNowWindow.onEscape = cancelFunc

    okButton.onClick = buyFunc
    cancelButton.onClick = cancelFunc
    MainWindow:hide()
    BuyNowWindow:show()
end

function onConfirmSellItem()
    local buffer = {
        protocol = "sell_item",
        itemid = SellPanelWindow:getChildById("sell_item").item.itemid,
        item_code = SellPanelWindow:getChildById("sell_item").item.item_code,
        count = SellPanelWindow:getChildById("sell_amount"):getValue(),
        maxtime = SellPanelWindow:getChildById("sell_timebox"):getText(),
        price = tonumber(SellPanelWindow:getChildById("sell_priceedit"):getText()) or 0,
        anonymous = SellPanelWindow:getChildById("sell_anonymous"):isChecked() and 1 or 0
    }
    for id, _time in ipairs(Market.time) do
        if _time.name == SellPanelWindow:getChildById("sell_timebox"):getText() then
            buffer.maxtime = id
            break
        end
    end
    g_game.getProtocolGame():sendExtendedOpcode(Market.opcode, json.encode(buffer))
end

function selectItemToSell()
    local gameInterface = modules.game_interface
    local mouseGrabberWidget = gameInterface.getMouseGrabberWidget()
    mouseGrabberWidget:grabMouse()
    g_mouse.pushCursor("target")
    mouseGrabberWidget.onMouseRelease = function(self, mousePosition, mouseButton)
        if mouseButton == MouseLeftButton then
            local clickedWidget = gameInterface.getRootPanel():recursiveGetChildByPos(mousePosition, false)
            if clickedWidget and clickedWidget:getClassName() == "UIItem" then
                g_game.getProtocolGame():sendExtendedOpcode(
                    Market.opcode,
                    json.encode({protocol = "selectItemToSell", pos = clickedWidget:getItem():getPosition()})
                )
                SellPanelWindow:focus()
            end
        end
        while (g_mouse.isCursorChanged()) do
            g_mouse.popCursor("target")
        end
        MainWindow:ungrabKeyboard()
        self:ungrabMouse()
        mouseGrabberWidget.onMouseRelease = nil
        mouseGrabberWidget.onMouseRelease = gameInterface.onMouseGrabberRelease
    end
end

function showBuyPanel()
    SellPanel:hide()
    SellPanelWindow:hide()
    HistoricPanel:hide()
    getBuyItems()
    BuyPanel:show()
end

function showSellPanelWindow()
    SellPanelWindow:show()
    SellPanelWindow:focus()
end

function showSellPanel()
    BuyPanel:hide()
    HistoricPanel:hide()
    getSellItems()
    SellPanel:show()
    SellPanelWindow:hide()
end

function showHistoricPanel()
    BuyPanel:hide()
    SellPanel:hide()
    g_game.getProtocolGame():sendExtendedOpcode(Market.opcode, json.encode({protocol = "historic"}))
    HistoricPanel:show()
end

function syncMarketSellItems()
    for i, child in ipairs(SellPanel:getChildById("items"):getChildren()) do
        local _t = child.market_item.timeleft
        child:getChildById("timer"):setText(
            _t < 0 and "Expired" or
                string.format(
                    "%02d:%02d:%02d",
                    math.floor(_t / (60 * 60)),
                    math.floor((_t / 60) % 60),
                    math.floor(_t % 60)
                )
        )
        child.market_item.timeleft = child.market_item.timeleft - 1
    end
    SellPanel.event = scheduleEvent(syncMarketSellItems, 1000)
end

function syncMarketBuyItems()
    for i, child in ipairs(BuyPanel:getChildById("items"):getChildren()) do
        local _t = child.market_item.timeleft
        child:getChildById("timer"):setText(
            _t < 0 and "Expired" or
                string.format(
                    "%02d:%02d:%02d",
                    math.floor(_t / (60 * 60)),
                    math.floor((_t / 60) % 60),
                    math.floor(_t % 60)
                )
        )
        child.market_item.timeleft = child.market_item.timeleft - 1
    end
    BuyPanel.event = scheduleEvent(syncMarketBuyItems, 1000)
end

function parseMarket(protocol, opcode, buffer)
    local receive = buffer
    if receive.protocol == "close" then
        MainWindow:hide()
        MainWindow:ungrabKeyboard()
    elseif receive.protocol == "show" then
        getBuyItems()
    elseif receive.protocol == "buy_items" then
        if not BuyPanel.event then
            BuyPanel.event = syncMarketBuyItems()
        end
        if receive.first then
            Current.page = receive.page
            Current.category = receive.category
            Current.max_page = math.max(1, receive.max_page)
            Current.search_string = receive.search_string
            updatePageWidget()
            BuyPanel:getChildById("items"):destroyChildren()
            BuyPanel:getChildById("buyButton"):setEnabled(false)
        end
        local child_count = #BuyPanel:getChildById("items"):getChildren()
        local function onFocus(widget, market_item, focused)
            if focused then
                widget:setBorderColor("red")
                BuyPanel:getChildById("buyButton"):setEnabled(true)
                Current.buyWidget = widget
            else
                widget:setBorderColor("white")
            end
        end
        for n, market_item in ipairs(receive.market_items) do
            local widget = g_ui.createWidget("MarketItem", BuyPanel:getChildById("items"))
            widget.market_item = market_item
            widget:getChildById("time"):setText("#" .. child_count)
            child_count = child_count + 1
            local _t = market_item.timeleft
            widget:getChildById("timer"):setText(
                _t < 0 and "Expired" or
                    string.format(
                        "%02d:%02d:%02d",
                        math.floor(_t / (60 * 60)),
                        math.floor((_t / 60) % 60),
                        math.floor(_t % 60)
                    )
            )
            widget.market_item.timeleft = widget.market_item.timeleft - 1
            widget:getChildById("item"):setItemId(market_item.clientId)
            widget:getChildById("item"):setItemCount(market_item.count)
            widget:getChildById("item"):setTooltip(market_item.description)
            if market_item.item_name:len() > 26 then
                widget:getChildById("name"):setText(
                    market_item.item_name:sub(0, -(market_item.item_name:len() + 1) + 25) .. "..."
                )
            else
                widget:getChildById("name"):setText(market_item.item_name)
            end
            widget:getChildById("name"):setColor(market_item.color)
            if market_item.anonymous == 1 then
                widget:getChildById("seller"):setText("Anonymous")
            else
                widget:getChildById("seller"):setText(market_item.seller_name)
            end
            widget:getChildById("amount"):setText(market_item.count)
            widget:getChildById("price"):setText("$" .. comma_value(market_item.price))
            widget.onFocusChange = function(reasonLabel, focused)
                onFocus(widget, market_item, focused)
            end
            if n == #receive.market_items then
                onFocus(widget, market_item, true)
            end
        end
        SellPanelWindow:hide()
        SellPanel:hide()
        HistoricPanel:hide()
        BuyPanel:show()
        MainWindow:show()
    elseif receive.protocol == "sell_items" then
        if not SellPanel.event then
            SellPanel.event = syncMarketSellItems()
        end
        if receive.first then
            SellPanel:getChildById("items"):destroyChildren()
            SellPanelWindow:getChildById("sell_item"):setItemId(0)
            SellPanelWindow:getChildById("sell_item"):setImageSource("/images/ui/item")
            SellPanelWindow:getChildById("sell_item"):setTooltip("")
            SellPanelWindow:getChildById("sell_itemname"):setText("")
            SellPanelWindow:getChildById("sell_amount"):setEnabled(false)
            SellPanelWindow:getChildById("sell_amount").onValueChange = nil
            SellPanelWindow:getChildById("sell_amount"):setValue(1)
            SellPanelWindow:getChildById("sell_amount"):setText("")
            SellPanelWindow:getChildById("sell_confirm"):setEnabled(false)
            SellPanelWindow:getChildById("sell_priceedit"):clearText()
            SellPanelWindow:getChildById("sell_priceedit"):setEnabled(false)
        end
        local child_count = #SellPanel:getChildById("items"):getChildren()
        local function onFocus(widget, market_item, focused)
            if focused then
                widget:setBorderColor("red")
            else
                widget:setBorderColor("alpha")
            end
        end
        for n, market_item in ipairs(receive.market_items) do
            local widget = g_ui.createWidget("MarketSellItem", SellPanel:getChildById("items"))
            widget.market_item = market_item
            widget:getChildById("time"):setText("#" .. child_count)
            child_count = child_count + 1
            local _t = market_item.timeleft
            widget:getChildById("timer"):setText(
                _t < 0 and "Expired" or
                    string.format(
                        "%02d:%02d:%02d",
                        math.floor(_t / (60 * 60)),
                        math.floor((_t / 60) % 60),
                        math.floor(_t % 60)
                    )
            )
            widget.market_item.timeleft = widget.market_item.timeleft - 1
            widget:getChildById("item"):setItemId(market_item.clientId)
            widget:getChildById("item"):setItemCount(market_item.count)
            widget:getChildById("item"):setTooltip(market_item.description)
            widget:getChildById("name"):setText(market_item.item_name)
            widget:getChildById("name"):setColor(market_item.color)
            widget:getChildById("amount"):setText(market_item.count)
            widget:getChildById("price"):setText("$" .. comma_value(market_item.price))
            widget.onFocusChange = function(reasonLabel, focused)
                onFocus(widget, market_item, focused)
            end
            if n == #receive.market_items then
                onFocus(widget, market_item, true)
            end
        end
    elseif receive.protocol == "item_to_sell" then
        SellPanelWindow:getChildById("sell_item").item = receive
        SellPanelWindow:getChildById("sell_item"):setItemId(receive.clientId)
        SellPanelWindow:getChildById("sell_item"):setTooltip(receive.description)
        SellPanelWindow:getChildById("sell_itemname"):setText(receive.item_name)
        SellPanelWindow:getChildById("sell_itemname"):setColor(receive.color)
        SellPanelWindow:getChildById("sell_amount"):setEnabled(true)
        SellPanelWindow:getChildById("sell_amount"):setValue(1)
        SellPanelWindow:getChildById("sell_amount"):setText(1)
        SellPanelWindow:getChildById("sell_amount"):setMinimum(1)
        SellPanelWindow:getChildById("sell_amount"):setMaximum(receive.count)
        SellPanelWindow:getChildById("sell_amount").onValueChange = function(self)
            local value = math.max(1, SellPanelWindow:getChildById("sell_amount"):getValue())
            SellPanelWindow:getChildById("sell_item"):setItemCount(value)
            SellPanelWindow:getChildById("sell_amount"):setText(value)
            onSellPriceChange()
        end
        SellPanelWindow:getChildById("sell_priceedit"):setEnabled(true)
        SellPanelWindow:getChildById("sell_priceedit"):clearText()
        SellPanelWindow:getChildById("sell_confirm"):setEnabled(true)
    elseif receive.protocol == "historic" then
        if receive.first then
            HistoricPanel:getChildById("list"):destroyChildren()
        end
        for num, h in ipairs(receive.historic) do
            local text = g_ui.createWidget("HistoricLabel", HistoricPanel:getChildById("list"))
            text:setId("historicwidget: " .. num)
            text:setText(" " .. h.negotiation)
        end
    end
end

function updatePageWidget()
    local pageWidget = BuyPanel:getChildById("pageWidget")
    pageWidget:getChildById("pageText"):setText(tr("Page") .. ": " .. Current.page .. "/" .. Current.max_page)
    pageWidget:getChildById("firstPage"):setEnabled(Current.page > 1 and true or false)
    pageWidget:getChildById("prevPage"):setEnabled(Current.page > 1 and true or false)
    pageWidget:getChildById("nextPage"):setEnabled(Current.page < Current.max_page and true or false)
    pageWidget:getChildById("lastPage"):setEnabled(Current.page < Current.max_page and true or false)
end

function getFormattedMoney2(value)
    local valueText = ""
    valueText = valueText .. value
    return "$" .. valueText
end

function getMarketFee(price)
    local fee = math.max(1, price * Market.feePercent / 100)
    return math.floor(fee)
end

function toggleOrder(sort)
    local currentSort = Current.order
    local newOrder = Current.order
    if not currentSort or currentSort == Market.order[sort .. "desc"] then
        newOrder = Market.order[sort .. "asc"]
    else
        newOrder = Market.order[sort .. "desc"]
    end

    refreshOrder(newOrder)
end

function refreshOrder(order)
    Current.order = order
    getBuyItems()
end

function closeMarket()
    MainWindow:hide()
    MainWindow:ungrabKeyboard()
	if SellPanelWindow then
		SellPanelWindow:hide()
	end
end
